'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var previewApi = require('@storybook/preview-api');
var clientLogger = require('@storybook/client-logger');
var defaultProjectAnnotations = require('@storybook/vue3/preview');

/** Function that sets the global config of your storybook. The global config is the preview module of your .storybook folder.
 *
 * It should be run a single time, so that your global config (e.g. decorators) is applied to your stories when using `composeStories` or `composeStory`.
 *
 * Example:
 *```jsx
 * // setup.js (for jest)
 * import { setProjectAnnotations } from '@storybook/testing-vue3';
 * import * as projectAnnotations from './.storybook/preview';
 *
 * setProjectAnnotations(projectAnnotations);
 *```
 *
 * @param projectAnnotations - e.g. (import * as projectAnnotations from '../.storybook/preview')
 */

function setProjectAnnotations(projectAnnotations) {
  previewApi.setProjectAnnotations(projectAnnotations);
}
/**
 *
 * @deprecated Use setProjectAnnotations instead
 */

function setGlobalConfig(projectAnnotations) {
  clientLogger.deprecate("setGlobalConfig is deprecated. Use setProjectAnnotations instead.");
  setProjectAnnotations(projectAnnotations);
}
/**
 * Function that will receive a story along with meta (e.g. a default export from a .stories file)
 * and optionally projectAnnotations e.g. (import * from '../.storybook/preview)
 * and will return a composed component that has all args/parameters/decorators/etc combined and applied to it.
 *
 *
 * It's very useful for reusing a story in scenarios outside of Storybook like unit testing.
 *
 * Example:
 *```jsx
 * import { render } from '@testing-library/vue3';
 * import { composeStory } from '@storybook/testing-vue3';
 * import Meta, { Primary as PrimaryStory } from './Button.stories';
 *
 * const Primary = composeStory(PrimaryStory, Meta);
 *
 * test('renders primary button with Hello World', () => {
 *   const { getByText } = render(Primary({ label: 'Hello world' }));
 *   expect(getByText(/Hello world/i)).not.toBeNull();
 * });
 *```
 *
 * @param story
 * @param componentAnnotations - e.g. (import Meta from './Button.stories')
 * @param [projectAnnotations] - e.g. (import * as projectAnnotations from '../.storybook/preview') this can be applied automatically if you use `setProjectAnnotations` in your setup files.
 * @param [exportsName] - in case your story does not contain a name and you want it to have a name.
 */

function composeStory(story, componentAnnotations, projectAnnotations, exportsName) {
  return previewApi.composeStory(story, componentAnnotations, projectAnnotations, defaultProjectAnnotations, exportsName);
}
/**
 * Function that will receive a stories import (e.g. `import * as stories from './Button.stories'`)
 * and optionally projectAnnotations (e.g. `import * from '../.storybook/preview`)
 * and will return an object containing all the stories passed, but now as a composed component that has all args/parameters/decorators/etc combined and applied to it.
 *
 *
 * It's very useful for reusing stories in scenarios outside of Storybook like unit testing.
 *
 * Example:
 *```jsx
 * import { render } from '@testing-library/vue3';
 * import { composeStories } from '@storybook/testing-vue3';
 * import * as stories from './Button.stories';
 *
 * const { Primary, Secondary } = composeStories(stories);
 *
 * test('renders primary button with Hello World', () => {
 *   const { getByText } = render(Primary({ label: 'Hello world' }));
 *   expect(getByText(/Hello world/i)).not.toBeNull();
 * });
 *```
 *
 * @param csfExports - e.g. (import * as stories from './Button.stories')
 * @param [projectAnnotations] - e.g. (import * as projectAnnotations from '../.storybook/preview') this can be applied automatically if you use `setProjectAnnotations` in your setup files.
 */

function composeStories(csfExports, projectAnnotations) {
  var composedStories = previewApi.composeStories(csfExports, projectAnnotations, composeStory);
  return composedStories;
}

exports.composeStories = composeStories;
exports.composeStory = composeStory;
exports.setGlobalConfig = setGlobalConfig;
exports.setProjectAnnotations = setProjectAnnotations;
//# sourceMappingURL=testing-vue3.cjs.development.js.map
